/**   -=- DrawLines -=-
 *
 *    This program creates thumbnail sketches from coordinates of lines
 *    recorded in a log file generated during user experiments. From a
 *    graphical user interface, this program allows to load the log file,
 *    and to browse the lines viewing the result on an image. This thumbnail
 *    image can be saved in a file under the PNG format. An accumulation
 *    mode allows to add several lines on the same thumbnail image.
 *
 *    Copyright (C) 2006-2008 Guillaume RIVIERE
 *
 *    This file is part of DrawLines.
 *
 *    DrawLines is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 * 
 *    DrawLines is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>
 **/


/**
 * Class Draw
 * - - - - - -
 * Une zone de dessin qui herite de DrawingArea.
 * 
 * Inspire des exemples suivants :
 *    http://www-lil.univ-littoral.fr/~quesnel/cplusplus/gtkmm/gtkmmse7.html
 *    http://www.sm.u-bordeaux2.fr/%7Eparisy/C++/gtkmm-exemples/exemples-gtkmm.tgz (lunarPhases)
 *
 * Guillaume Riviere 21-06-2006
 */

#include <iostream>

#include <gtkmm/drawingarea.h>

#include "Draw.hh" 

using namespace std ;

/////////////////////////////////////////
//-- Constructeurs / Destructeur PUBLIC
/////////////////////////////////////////

Draw::Draw(int w, int h)
  : Gtk::DrawingArea ()
{  
  /* Set the minimum size of the widget */
  set_size_request(w, h);  
}

//////////////////////
//-- Methodes PUBLIC
//////////////////////

void Draw::set_image_from_file (const char* filename)
{
  m_pixbuf = Gdk::Pixbuf::create_from_file(filename) ;
}

/////////////////////////
//-- Methodes PROTECTED
/////////////////////////

/**
 * Dessine une image dans la zone de dessin a partir d'un fichier.
 */
void Draw::draw_image ()
{
  int x = 0 ;
  int y = 0 ;
  int width = m_pixbuf->get_width () ;
  int height = m_pixbuf->get_height () ;

  //cout << "pixbuf size = " << width << " x " << height << endl ;

  this->set_size_request (width, height) ;

  const int rowstride = m_pixbuf->get_rowstride();
  const size_t offset = rowstride * y + 3 * x;

  this->get_window()->draw_rgb_image_dithalign(this->get_style()->get_black_gc(),
					       x, y, width, height,
					       Gdk::RGB_DITHER_NORMAL,
					       &m_pixbuf->get_pixels()[offset], rowstride,
					       x, y);

}

/**
 * Redessine la zone lors de l'appel d'vnement expose_event.
 */
bool Draw::on_expose_event(GdkEventExpose* event)  
{  
  /* Draw image */
  this->draw_image () ;

  return true;  
}  
