/**   -=- DrawLines -=-
 *
 *    This program creates thumbnail sketches from coordinates of lines
 *    recorded in a log file generated during user experiments. From a
 *    graphical user interface, this program allows to load the log file,
 *    and to browse the lines viewing the result on an image. This thumbnail
 *    image can be saved in a file under the PNG format. An accumulation
 *    mode allows to add several lines on the same thumbnail image.
 *
 *    Copyright (C) 2006-2008 Guillaume RIVIERE
 *
 *    This file is part of DrawLines.
 *
 *    DrawLines is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 * 
 *    DrawLines is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>
 **/


/**
 * Class Draw
 * - - - - - -
 * Une zone de dessin qui herite de DrawingArea.
 * 
 * Inspire des exemples suivants :
 *    http://www-lil.univ-littoral.fr/~quesnel/cplusplus/gtkmm/gtkmmse7.html
 *    http://www.sm.u-bordeaux2.fr/%7Eparisy/C++/gtkmm-exemples/exemples-gtkmm.tgz (lunarPhases)
 *
 * Guillaume Riviere 21-06-2006
 */

#ifndef DRAW_H
#define DRAW_H

#include <vector>

#include <gtkmm/drawingarea.h>


class Draw : public Gtk::DrawingArea  
{  
public:  

  //-- Constructeurs / Destructeur PUBLIC

  Draw (int w, int h);

  //-- Methodes PUBLIC

  void set_image_from_file (const char* filename) ;

protected:

  //-- Methodes PROTECTED

  void draw_image () ;

 /**
   * Methode heritee de Gtk::Widget
   */
  bool on_expose_event(GdkEventExpose* event);

  //-- Attributs PROTECTED

  Glib::RefPtr<Gdk::Pixbuf> m_pixbuf ;

private:

  //-- Attributs PRIVATE

};  

#endif /* DRAW_H */
