/**   -=- DrawLines -=-
 *
 *    This program creates thumbnail sketches from coordinates of lines
 *    recorded in a log file generated during user experiments. From a
 *    graphical user interface, this program allows to load the log file,
 *    and to browse the lines viewing the result on an image. This thumbnail
 *    image can be saved in a file under the PNG format. An accumulation
 *    mode allows to add several lines on the same thumbnail image.
 *
 *    Copyright (C) 2006-2008 Guillaume RIVIERE
 *
 *    This file is part of DrawLines.
 *
 *    DrawLines is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 * 
 *    DrawLines is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>
 **/


#ifndef DRAWLINES_H
#define DRAWLINES_H

#include <vector>
#include <string>

using namespace std ;

#include "Draw.hh"
#include "cut.hh"

struct line {
  int id ;
  int x1, y1 ;
  int x2, y2 ;
  int X1, Y1 ;
  int X2, Y2 ;
} ;

class DrawLines : public Draw
{  
public:  

  //-- Constructeurs / Destructeur PUBLIC

  DrawLines (int w, int h);

  //-- Accesseurs PUBLIC

  inline int getWidth () const
  { return m_pixbuf->get_width() ; }

  inline int getHeight () const
  { return m_pixbuf->get_height() ; }

  inline void line_expected_enable (const bool enable)
  { m_line_expected_isDrawing = enable ; }

  inline void line_enable (const bool enable)
  { m_isDrawing = enable ; }

  void set_map_coord (const double Xmin, const double Xmax,
		      const double Ymin, const double Ymax) ;

  void set_picture_coord (const double Xmin, const double Xmax,
			  const double Ymin, const double Ymax) ;

  void set_draw_image_border (const bool active)
  {  m_draw_image_border = active ; }

  //-- Methodes PUBLIC

  inline void set_ex (const int ex)
  {
    m_cut.ex = ex ;
  }

  inline int set_line (const double x1, const double y1,
		       const double x2, const double y2)
  {
    m_cut.x1 = x1 ;
    m_cut.y1 = y1 ;
    m_cut.x2 = x2 ;
    m_cut.y2 = y2 ;
  }

  void save_to_file (const string& filename) ;

  void multiline_active (const bool active) ;
  void multiline_add (const double x1, const double y1,
		      const double x2, const double y2) ;
  void multiline_clear () ;

protected:

  int x2graph (const double x) ;

  int y2graph (const double y) ;

  //-- Methodes PROTECTED

 /**
   * Methode heritee de la classe Draw
   */
  bool on_expose_event(GdkEventExpose* event);


  void queue_draw_area_optimizedForLine (double xA, double yA, double xB, double yB, int n) ;

  bool equation_affine (double x1, double y1,
			double x2, double y2,
			double* a, double* b) ;

  void intersections_with_area (double bot, double top,
				double left, double right,
				double a, double b,
				int* x1, int* y1,
				int* x2, int* y2) ;

private:

  //-- Attributs PRIVATE

  /* Range (Taille reelle de la carte) */
  double m_Xmin ;
  double m_Ymin ;
  double m_Xmax ;
  double m_Ymax ;

  /* Map (Carte miniaturisee) */
  double m_map_Xmin ;
  double m_map_Ymin ;
  double m_map_Xmax ;
  double m_map_Ymax ;

  /* - - - - - - - - - - - - */

  bool m_isDrawing ;

  bool m_draw_image_border ;

  bool m_line_expected_isDrawing ;

  bool m_multiline_active ;

  /* - - - - - - - - - - - - */

  struct cut m_cut ;

  std::vector<struct cut> m_multiline_tab ;
};  


#endif /* DRAWLINES_H */
