/**   -=- DrawLines -=-
 *
 *    This program creates thumbnail sketches from coordinates of lines
 *    recorded in a log file generated during user experiments. From a
 *    graphical user interface, this program allows to load the log file,
 *    and to browse the lines viewing the result on an image. This thumbnail
 *    image can be saved in a file under the PNG format. An accumulation
 *    mode allows to add several lines on the same thumbnail image.
 *
 *    Copyright (C) 2006-2008 Guillaume RIVIERE
 *
 *    This file is part of DrawLines.
 *
 *    DrawLines is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 * 
 *    DrawLines is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>
 **/


// generated 2006/11/8 10:52:58 CET by guillaume@pc-ludo-max2.(none)
// using glademm V2.6.0
//
// newer (non customized) versions of this file go to Window1.cc_new

// This file is for your program, I won't touch it again!

#include <iostream>
#include <math.h>

#include "config.h"
#include "Window1.hh"

#include <sstream>
#include <string>
#include <ostream>
#include <fstream>

using namespace std ;

template <typename T>

bool fromString(const std::string &s, T &result)
{
  istringstream stream (s);
  return (stream >> result) ;
}

char *itoa(int n, char *s) {
  if (n>=10)
    s = itoa(n/10, s) ;

  *s++ = '0' + (n % 10) ;
  *s = '\0' ;

  return s ;
}

string intToString (const int n)
{
  char* str = new char [(n<1) ? 1 : (int)floor(log10((double)n)) + 1] ;

  (void) itoa (n, str) ;

  string res (str) ;

  delete [] str ;

  return res ;
}

string toString (double x)
{
  std::ostringstream o;
  o << x ;
  return o.str();
}

//////////////////////////////////////////
//-- Constructeurs / Destructeurs PUBLIC
//////////////////////////////////////////

Window1::Window1 ()
  : Window1_glade (),
    m_zoneDessin (NULL)
{
  /* --- Zone de dessin --- */

  /* Creation du zone de dessin de la classe Draw (herite de Gtk::DrawingArea) */
  m_zoneDessin = Gtk::manage (new class DrawLines (150, 132)) ;

  /* Placement de la zone de dessin */
  vbox5->pack_start(*m_zoneDessin);

  /* Le widget est pret */
  m_zoneDessin->show() ;

  /* Chargement d'une image */
  m_zoneDessin->set_image_from_file ("white-132.jpg") ;

  /* Raffraichissement de l'affichage (Ask for an expose event) */
  queue_draw () ;

  /* --------------------- */

  set_map_coord (60, 380, 40, 356) ;
  m_zoneDessin->set_map_coord (68, 380, 40,356) ;
  m_zoneDessin->set_picture_coord (0, 450, 0, 398) ;

  m_zoneDessin->set_draw_image_border (true) ;
  check_contour_image->set_active(true) ;

  check_cumuler_lignes->set_active(false) ;
  m_zoneDessin->multiline_active (false) ;

  /* --------------------- */

  m_radio_tab[0] = rb_ex1 ;
  m_radio_tab[1] = rb_ex2 ;
  m_radio_tab[2] = rb_ex3 ;
  m_radio_tab[3] = rb_ex4 ;
  m_radio_tab[4] = rb_ex5 ;
  m_radio_tab[5] = rb_ex6 ;
  m_radio_tab[6] = rb_ex7 ;
}

/////////////////////////
//-- Accesseurs PRIVATE
/////////////////////////

void Window1::set_cut_pos (const int i)
{
  struct cut *c = m_cuts_tab[i] ;
  m_cut_pos = i ;

  entry_cut_pos->set_text (intToString (i).c_str()) ;
  entry_cut_name->set_text (c->name.c_str()) ;
  entry_cut_pos->queue_draw() ;
  entry_cut_name->queue_draw() ;

  /* Activer bouton prec */
  if (m_cut_pos == 0)
    bouton_prec->set_sensitive (false) ;
  else
    bouton_prec->set_sensitive (true) ;

  /* Activer bouton suiv */
  if (m_cut_pos == m_cuts_tab.size()-1)
    bouton_suiv->set_sensitive (false) ;
  else
    bouton_suiv->set_sensitive (true) ;

  /* MAJ presentation */
  entry_x1->set_text (toString(c->x1)) ;
  entry_y1->set_text (toString(c->y1)) ;
  entry_x2->set_text (toString(c->x2)) ;
  entry_y2->set_text (toString(c->y2)) ;

  m_radio_tab[c->ex]->set_active (true) ;

  /* MAJ zoneDessin */
  m_zoneDessin->set_ex (c->ex) ;
  m_zoneDessin->set_line (c->x1, c->y1, c->x2, c->y2) ;

  m_zoneDessin->line_enable (true) ;
  m_zoneDessin->queue_draw () ;
}

void Window1::set_map_coord (const double Xmin, const double Xmax,
			     const double Ymin, const double Ymax)
{
  entry_map_Xmin->set_text (toString(Xmin)) ;
  entry_map_Xmax->set_text (toString(Xmax)) ;

  entry_map_Ymin->set_text (toString(Ymin)) ;
  entry_map_Ymax->set_text (toString(Ymax)) ;
}

////////////////////////
//-- Callbacks PRIVATE
////////////////////////

void Window1::on_rb_ex1_clicked()
{
  if (rb_ex1->get_active()) {
    m_zoneDessin->set_ex (0) ;
    m_zoneDessin->queue_draw () ;
  }
}

void Window1::on_rb_ex2_clicked()
{
  if (rb_ex2->get_active()) {
    m_zoneDessin->set_ex (1) ;
    m_zoneDessin->queue_draw () ;
  }
}

void Window1::on_rb_ex3_clicked()
{
  if (rb_ex3->get_active()) {
    m_zoneDessin->set_ex (2) ;
    m_zoneDessin->queue_draw () ;
  }
}

void Window1::on_rb_ex4_clicked()
{
  if (rb_ex4->get_active()) {
    m_zoneDessin->set_ex (3) ;
    m_zoneDessin->queue_draw () ;
  }
}

void Window1::on_rb_ex5_clicked()
{
  if (rb_ex5->get_active()) {
    m_zoneDessin->set_ex (4) ;
    m_zoneDessin->queue_draw () ;
  }
}

void Window1::on_rb_ex6_clicked()
{
  if (rb_ex6->get_active()) {
    m_zoneDessin->set_ex (5) ;
    m_zoneDessin->queue_draw () ;
  }
}

void Window1::on_rb_ex7_clicked()
{
  if (rb_ex7->get_active()) {
    m_zoneDessin->set_ex (6) ;
    m_zoneDessin->queue_draw () ;
  }
}

void Window1::on_check_contour_image_toggled()
{
  m_zoneDessin->set_draw_image_border (check_contour_image->get_active()) ;
  m_zoneDessin->queue_draw () ;
}

void Window1::on_check_cumuler_lignes_toggled()
{
  bool active = check_cumuler_lignes->get_active() ;

  m_zoneDessin->multiline_active (active) ;
  bouton_ajouter_ligne->set_sensitive (active) ;

  if (active)
    m_zoneDessin->multiline_clear () ;

  m_zoneDessin->queue_draw () ;
}

void Window1::on_bouton_valider_clicked()
{

  double x1, y1, x2, y2 ;

  fromString (entry_x1->get_text (), x1) ;
  fromString (entry_y1->get_text (), y1) ;
  fromString (entry_x2->get_text (), x2) ;
  fromString (entry_y2->get_text (), y2) ;

  m_zoneDessin->set_line (x1, y1, x2, y2) ;

  m_zoneDessin->line_enable (true) ;

  m_zoneDessin->queue_draw () ;
}

void Window1::on_bouton_sauver_clicked()
{
  m_zoneDessin->save_to_file ("Images/" + filename->get_text () + ".png") ;
}

void Window1::on_bouton_prec_clicked()
{
  set_cut_pos (m_cut_pos-1) ;
}

void Window1::on_bouton_suiv_clicked()
{
  set_cut_pos (m_cut_pos+1) ;
}

void Window1::on_entry_cut_pos_changed()
{
  int pos ;
  if (fromString(entry_cut_pos->get_text(), pos) && pos >= 0)
    set_cut_pos (pos) ;
}

void Window1::on_button_sauver_cut_clicked()
{
  m_zoneDessin->save_to_file ("Images/" + entry_cut_name->get_text () + ".png") ;

  if (m_cut_pos+1 < m_cuts_tab.size())
    set_cut_pos (m_cut_pos+1) ;
}

void Window1::on_bouton_ajouter_ligne_clicked()
{
  double x1, y1, x2, y2 ;

  fromString (entry_x1->get_text (), x1) ;
  fromString (entry_y1->get_text (), y1) ;
  fromString (entry_x2->get_text (), x2) ;
  fromString (entry_y2->get_text (), y2) ;

  m_zoneDessin->multiline_add (x1, y1, x2, y2) ;

  if (m_cut_pos+1 < m_cuts_tab.size())
    set_cut_pos (m_cut_pos+1) ;

  m_zoneDessin->queue_draw () ;
}

void Window1::on_bouton_charger_clicked()
{
  ifstream in (entry_cutsfile->get_text().c_str(), ios::in) ;
  string line ;
  struct cut *c ;

  /* Verification ouverture fichier */
  if (!in) {
    cerr << "** configuration_read: Erreur d'ouverture en lecture du fichier " << filename << endl ;
    return ;
  }

  /* Vider memoire */
  for (int i=0 ; i < m_cuts_tab.size() ; i++)
    delete m_cuts_tab[i] ;
  m_cuts_tab.clear () ;

  /* Lecture fichier */
  while (!in.eof()) {

    getline (in, line) ;

    /* Verifier que ce n'est pas une ligne de commentaire */
    if (line.length() > 1 && line[0] != '#' && line[0] != ' ') {

      c = new struct cut ;

      istringstream stream (line);

      stream >> c->name ;
      stream >> c->ex ;

      stream >> c->x1 ;
      stream >> c->y1 ;
      stream >> c->x2 ;
      stream >> c->y2 ;

      cout << "| name " << c->name << "  ex " << c->ex ;
      cout << "   " << c->x1 << " " << c->y1 << " " << c->x2 << " " << c->y2 ;

      cout << endl ;

      m_cuts_tab.push_back (c) ;
    }
  }
  
  cout << "Loaded." << endl ;

  set_cut_pos (0) ;

  entry_cut_pos->set_sensitive (true) ;
}

void Window1::on_bouton_dessiner_carte_clicked()
{
  double xmin, xmax ;
  double ymin, ymax ;

  fromString (entry_map_Xmin->get_text (), xmin) ;
  fromString (entry_map_Xmax->get_text (), xmax) ;

  fromString (entry_map_Ymin->get_text (), ymin) ;
  fromString (entry_map_Ymax->get_text (), ymax) ;  

  m_zoneDessin->set_map_coord (xmin, xmax, ymin, ymax) ;
}
