/**   -=- DrawLines -=-
 *
 *    This program creates thumbnail sketches from coordinates of lines
 *    recorded in a log file generated during user experiments. From a
 *    graphical user interface, this program allows to load the log file,
 *    and to browse the lines viewing the result on an image. This thumbnail
 *    image can be saved in a file under the PNG format. An accumulation
 *    mode allows to add several lines on the same thumbnail image.
 *
 *    Copyright (C) 2006-2008 Guillaume RIVIERE
 *
 *    This file is part of DrawLines.
 *
 *    DrawLines is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 * 
 *    DrawLines is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>
 **/


#include <math.h>
#include <sstream>

#include "my_string.h"
#include "my_itoa.h"

/**
 * Convertir une chaine de caracteres en entier.
 *
 * @param str La chaine a convertir.
 * @return int L'entier.
 */
int stringToInt (const string& str)
{
  istringstream iss (str) ;
  int res ;
  iss >> res ;
  return res ;
}

/**
 * Convertir un entier en chaine de caracteres.
 *
 * @param int n L'entier a convertir.
 * @return int La chaine de caracteres.
 */
string intToString (const int n)
{
  char* str = new char [(n<1) ? 1 : (int)floor(log10((double)n)) + 1] ;

  (void) itoa (n, str) ;

  string res (str) ;

  delete [] str ;

  return res ;
}
